<?php

use LMS\Libraries;
use LMS\Libraries\ServiceLocator;

class JFormFieldModalStudentsList extends JFormField
{	
	protected $type = 'StudentsListModal';
	
	protected function getInput()
	{	
		ServiceLocator::load('tool.users');		
	
		$db = ServiceLocator::get('db');
		
		$assets = ServiceLocator::get('assets');		
		$assets->addScript('tools/users/assets/js/fields/studentslistmodal.js');
		
		$this->config	= ServiceLocator::get('config');		
		$courseId = $this->config->get('course_id');
		
		$students = array();		
		$groups = array();
		$teachers = array();		
		$studentsNames = array();
		$staffNames = array();
		$groupsNames = array();
		$excludedStudents = array();
		$excludedGroups = array();
		$excludedStaff = array();
		
		if($this->value) {
			if(is_array($this->value)) {
				if(
					isset($this->value['students']) || 
					isset($this->value['groups']) || 
					isset($this->value['teachers']) || 
					isset($this->value['excluded_students']) || 
					isset($this->value['excluded_groups']) || 
					isset($this->value['excluded_staff'])
				){
					$students	= isset($this->value['students'])?$this->value['students']:array();
					$groups		= isset($this->value['groups'])?$this->value['groups']:array();
					$teachers	= isset($this->value['teachers'])?$this->value['teachers']:array();
					$excludedStudents = isset($this->value['excluded_students'])?$this->value['excluded_students']:array();
					$excludedGroups = isset($this->value['excluded_groups'])?$this->value['excluded_groups']:array();
					$excludedStaff = isset($this->value['excluded_staff'])?$this->value['excluded_staff']:array();
				} else {
					$rep = ServiceLocator::get('student_rep');
					$students = $rep->getByFilter(array(
						'fields'=>'student.user_id',
						'course_id'=>$courseId,					
						'user_id'=>$recipients,
						'resultType'=>'column'
					));
					
					$teachers = array_diff($recipients, $students);
				}
			} else {			
				$rep = ServiceLocator::get('student_rep');				
				$recipient = (int)$this->value;
				
				$isStudent = $rep->getByFilter(array(
					'fields'=>'COUNT(student.user_id)',
					'course_id'=>$courseId,
					'user_id'=> $recipient,
					'resultType'=> 'result'
				));
				
				if($isStudent) {
					$students = array($recipient);
				} else {
					$teachers = array($recipient);
				}
			}
			
			$rep = ServiceLocator::get('student_rep');
		
			$studentsNames = $rep->getByFilter(array(
				'fields'=>'DISTINCT student.profile.name',
				'course_id'=>$courseId,
				'user_id'=> $students,
				'resultType'=>'column'
			));
			
			$rep = ServiceLocator::get('group_rep');
		
			$groupsNames = $rep->getByFilter(array(
				'fields'=>'DISTINCT group.name',				
				'group_id'=> $groups,
				'resultType'=>'column'
			));			
				
			$options = array('user_id'=> $teachers);
		
			$repositoryStaff = ServiceLocator::get('staff_rep');
			$repositoryCourseStaff	= ServiceLocator::get('coursestaff_rep');
			
			$options['fields'] = 'staff.profile.name';
			
			$query = $repositoryStaff->getByFilterQuery($options);
			
			$options['fields'] = 'coursestaff.profile.name';
			
			$sub = $repositoryCourseStaff->getByFilterQuery($options);		
			
			$query->union($sub);	
			
			$db->setQuery($query);
			
			$staffNames = $db->loadColumn();
		}
		
		$countStudents = ($students)?count($students):0;
		
		$this->addGroup($courseId, $students, $excludedStudents);
		
		$students = (object)$students;
		$excludedStudents = (object)$excludedStudents;
			
		$value = json_encode(array(
			"students" => $students, 
			"groups" => $groups, 
			"staff" => $teachers, 
			"excludedStudents" => $excludedStudents,
			"excludedGroups" => $excludedGroups,
			"excludedStaff" => $excludedStaff
		));

		$context = 'studentsmodal';
				
		$html = '			
			<a class="modal btn modalstudentlist-btn"
			id="'.$context.'-btn"
			href="'.sefRelToAbs('index.php?option=com_joomla_lms&task=users.users.studentslistmodal&course_id='.$courseId).'" 
			rel="{handler:\'iframe\', size:{x:1024, y:500}}">[<span id="'.$context.'-btn-label">'.JText::_('_JLMS_CHOOSE').'</span><i class="icon-user"></i>]</a>';
					
		$html .= '<font '.(empty($groupsNames)?'class="modalstudentlist-label hidden"':'').' id="'.$context.'-groups" color="red"> '.JText::_('_JLMS_GROUPS').':</font>';
		$html .= '<span class="groupsmodal-names">'.(($groupsNames == '*')?JText::_('_JLMS_ALL'):implode(' ', $groupsNames)).'</span>';
		$html .= '<font '.(empty($staffNames)?'class="modalstudentlist-label hidden"':'').' id="'.$context.'-staff" color="green"> '.JText::_('_JLMS_STAFF').':</font>';
		$html .= '<span class="staffmodal-names"> '.(($staffNames == '*')?JText::_('_JLMS_ALL'):implode(' ', $staffNames)).'</span>';
		$html .= '<font '.(empty($studentsNames)?'class="modalstudentlist-label hidden"':'').' id="'.$context.'-students" color="blue"> '.JText::_('_JLMS_STUDENTS').':</font>';
		$html .= '<span class="studentsmodal-names"> '.(($studentsNames == '*')?JText::_('_JLMS_ALL'):implode(' ', $studentsNames)).'</span>';	
		
		$html .= '<input type="hidden" name="'.$this->name.'" class="validate-studentslistmodal" id="'.$this->id.'" value="'.htmlentities($value).'"/>';

		return $html;			
	}
	
	protected function addGroup($courseId, & $students, & $excluded)
	{			
		if(empty($excluded) && empty($students)) {
			return false;
		}
		
		//ServiceLocator::load('lib.form.fields');
		$rep	= ServiceLocator::get('student_rep');

		$rows = $rep->getByFilter(array(
			'fields'=>'DISTINCT student.user_id, student.studentgroup.group_id',
			'course_id'=>$courseId,
			'user_id'=> array_merge($students, $excluded)
		));
		
		$stuArr = $students;
		
		$students = array();
		$excluded = array();
		
		foreach($rows AS $row) {
			if(in_array($row->user_id, $stuArr)) {
				if(!isset($students[$row->user_id])) {
					$students[$row->user_id] = array();
				}
				$students[$row->user_id][] = (int)$row->group_id;
			} else {
				if(!isset($excluded[$row->user_id])) {
					$excluded[$row->user_id] = array();
				}
				
				$excluded[$row->user_id][] = (int)$row->group_id;
			}
		}		
	}
	
	public function __get($name)
	{		
		if($name == 'value') 
		{			
			return $this->handleValue($this->value);
		} else if($name == 'valueNames') {
			return $this->valueNames();
		}else {
			return parent::__get($name);
		}	
	}	
	
	protected function handleValue($data)
	{			
		$config = ServiceLocator::get('config');
		
		$db = ServiceLocator::get('db');

		ServiceLocator::load('lib.form.fields');		
		ServiceLocator::load('tool.users');
		
		$courseId = $config->get('course_id');
		
		$recipients = json_decode($data);	
	
		$recipients->students = (array)$recipients->students;		
		$recipients->excludedStudents = (array)$recipients->excludedStudents;
		
		$starPosStudents = array_search('*', $recipients->students);
		$starPosGroups = array_search('*', $recipients->groups);
		$starPosStaff = array_search('*', $recipients->staff);
				
		$allStudentsSelected = $starPosStudents !== false;
		$allGroupsSelected	= $starPosGroups !== false;
		$allStaffSelected	= $starPosStaff !== false;
		
		/*removing star from array*/
		if($starPosStudents !== false ) {
			unset($recipients->students[$starPosStudents]);
			$recipients->students = array_values($recipients->students);
		}		
		
		if($starPosGroups !== false ) {
			unset($recipients->groups[$starPosGroups]);
			$recipients->groups = array_values($recipients->groups);
		}
		
		if($starPosStaff !== false ) {
			unset($recipients->staff[$starPosStaff]);
			$recipients->staff = array_values($recipients->staff);
		}
						
		$selectedStudents	= array_map('intval', array_keys($recipients->students));		
		$selectedGroups		= array_map('intval', $recipients->groups);
		$selectedStaff		= array_map('intval', $recipients->staff);		
		$excludedStudents	= array_map('intval', array_keys($recipients->excludedStudents));
		$excludedGroups		= array_map('intval', $recipients->excludedGroups);
		$excludedStaff		= array_map('intval', $recipients->excludedStaff);		

		$studentsList	= array();
		$staffList		= array();
		$options	= array(
			'fields'=>'DISTINCT student.user_id',			
			'user_or_group'=> true,
			'course_id' => $courseId,
			'resultType'=>'column'
		);		

		$rep = ServiceLocator::get('student_rep');	
		
		if($allStudentsSelected) {			
			if(!empty($excludedStudents)) {
				$options['excluded_user_id'] = $excludedStudents;
			} 
		} else {
			$students = array_diff($selectedStudents, $excludedStudents);			
			$options['user_id'] = $students;
		}		
		
		if($allGroupsSelected) {			
			if(!empty($excludedGroups)) {
				$options['excluded_group_id'] = $excludedGroups;
			} 
		} else {				
			$groups = array_diff($selectedGroups, $excludedGroups);			
			$options['group_id'] = $groups;
		}		
		
		$studentsList = $rep->getByFilter($options);		
		
		$options = array(
				'fields'=>'DISTINCT staff.user_id',					
				'resultType'=>'column'				
		);
		
		if($allStaffSelected) {			
			if(!empty($excludedStaff)) {
				$options['excluded_user_id'] = $excludedStaff;
			}
		} else {
			$staff = array_diff($selectedStaff, $excludedStaff);
			$options['user_id'] = $staff;
		}	
		
		$repositoryStaff = ServiceLocator::get('staff_rep');
		$repositoryCourseStaff	= ServiceLocator::get('coursestaff_rep');
		
		$options['fields'] = 'staff.user_id';
		
		$query = $repositoryStaff->getByFilterQuery($options);
		
		$options['fields'] = 'coursestaff.user_id';
		$options['course_id'] = $courseId;
		
		$sub = $repositoryCourseStaff->getByFilterQuery($options);				
		
		$query->union($sub);	
		
		$db->setQuery($query);
		
		$staffList = $db->loadColumn();		
		
		$studentsList		= array_map('intval', $studentsList);
		$staffList		= array_map('intval', $staffList);
		
		$res = array_merge($studentsList, $staffList);					
		
		return $res;
	}
	
	protected function valueNames() 	
	{			
		$rep = ServiceLocator::get('user_rep');
		
		if(empty($this->value)) 
		{
			return '';
		}
		
		$names = $rep->getByFilter(array(
			'fields'=>'DISTINCT joomlauser.name',
			'user_id' => $this->value,
			'resultType'=>'column'
		));
		
		if(empty($names)) {
			$res = '';
		} else {
			$res = implode(', ', $names);
		}			
		
		return $res;
	}
	
}
