<?php
/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 *
 */

namespace LMS\Widgets;

use LMS\Libraries\Base;

class Button extends Base\Widget
{ 
	protected $items = array();
	
	public function __construct($params = array())
	{	
		parent::__construct($params);
		
		$this->glue		= isset($params['glue'])?$params['glue']:'';
		$this->size		= isset($params['size'])?$params['size']:'';		
		$this->type		= isset($params['type'])?$params['type']:'';
		$this->active	= isset($params['active'])?$params['active']:'';
		$this->disabled	= isset($params['disabled'])?$params['disabled']:'';
		$this->id		= isset($params['id'])?$params['id']:'';
		$this->href		= isset($params['href'])?str_replace('"', "'", $params['href']):'';
		$this->title	= isset($params['title'])?$params['title']:'';
		$this->task		= isset($params['task'])?$params['task']:'';
		$this->icon		= isset($params['icon'])?$params['icon']:'';
		$this->jlmsicon		= isset($params['jlmsicon'])?$params['jlmsicon']:'';
		$this->height	= isset($params['height'])?$params['height']:600;
		$this->width	= isset($params['width'])?$params['width']:600;
		$this->items	= isset($params['items'])?$params['items']:array();
		$this->click	= isset($params['click'])?str_replace('"', "'", $params['click']):'';
		$this->class	= isset($params['class'])?$params['class']:'';
		$this->tip		= isset($params['tip'])?$params['tip']:'';
		$this->iconPosition	= isset($params['iconPosition'])?$params['iconPosition']:'left';
		$this->tipDescription = isset($params['tipDescription'])?$params['tipDescription']:'';
		$this->target	= isset($params['target'])?$params['target']:'';
		$this->nowrap	= isset($params['nowrap'])?$params['nowrap']:'';
		
		if($this->size == 'micro') {
			$this->class .= ' btn-micro';
		}
		
		if($this->active) {
			$this->class .= ' active';
		}

		if($this->disabled) {
			$this->class .= ' disabled';
		}

		if($this->tip) {
			\JHtml::_('bootstrap.tooltip');
		}
		$this->published	= isset($params['published'])?$params['published']:true;

		switch($this->type) {			
			case 'custom':
				if(empty($params['href'])) {
					$this->setTemplate('button');					
				} else {
					$this->setTemplate('custom');
				}
			break;
			case 'modal':
				$this->setTemplate('modal');
			break;
			case 'submit':
				$this->setTemplate('submit');
			break;
			case 'button':
				$this->setTemplate('button');
			break;
			case 'separator':
				$this->setTemplate('separator');
			break;
			case 'export':				
			case 'dropdown':
				$this->setTemplate('dropdown');
			break;			
			case 'dropdown_custom':			
				$this->setTemplate('dropdown_custom');
				break;
			case 'dropdownpublished':
				$this->setTemplate('dropdownpublished');
				break;
			case 'dropdown_separator':
				$this->setTemplate('dropdown_separator');
			break;
			case 'dropdown_split':
				$this->setTemplate('dropdown_split');
				break;
			default:
		}
	}
	
	public function addCustomItem($params) 
	{
		$key = '';
		
		if(!in_array($this->type, array('dropdownpublished', 'dropdown_split'))){
			$this->type = 'dropdown';
			$this->setTemplate('dropdown');
		}
		
		if(isset($params['task'])) {
			$key = $params['task'];
		} else if(isset($params['href'])) {
			$key = $params['href'];
		}
		
		if(isset($params['title'])) {
			$key .= $params['title'];
		}		
		
		$alias = base64_encode($key);
		
		if(empty($params['type'])) {
			$params['type'] = 'custom';
		}

		$params['type'] = 'dropdown_'.$params['type'];
		
		$item = $this->widget('button', $params, $alias);
		
		$this->items[] = $item;
		
		return $item;
	}

	public function addExportItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'box-remove';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_EXPORT;
		}
		
		return $this->addCustomItem($params);
	}

	public function addImportItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'box-add';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_IMPORT;
		}
		
		return $this->addCustomItem($params);
	}
	
	public function addNewItem($params = array()) 
	{			
		$params['type'] = 'custom';
		$params['icon'] = 'plus';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_NEW;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}
	
	public function addToggleItem($params = array())
	{
		if(isset($params['statesassoc']) && isset($params['state'])) {			
			if(in_array($params['state'], array_keys($params['statesassoc']))) {
				$params = array_merge($params, $params['statesassoc'][$params['state']]);
			} 
		}
		
		if(!isset($params['type'])) {				
			$params['type'] = 'custom';
		}
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_PUBLISH;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}
	
	public function addTogglePublishItem($params = array())
	{
		if(!isset($params['statesassoc'])) {
			$params['statesassoc'] = array(
				0 => array('icon' => 'publish', 'title' => _JLMS_UNPUBLISH),
				1 => array('icon' => 'unpublish', 'title' => _JLMS_PUBLISH)
			);
		}
		
		$this->addToggleItem($params);
		
		return $this;
	}
	
	public function addEditItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'pencil';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_EDIT;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}
	
	public function addDeleteItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'trash';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_DELETE;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}
	
	public function addPublishItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'publish';		
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_PUBLISH;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}	
	
	public function addUnpublishItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'unpublish';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_UNPUBLISH;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}
	
	public function addCopyItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'copy';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_COPY;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}
	
	public function addMoveItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'move';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_MOVE;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}

	public function addPreviewItem($params = array()) 
	{
		$params['type'] = 'custom';
		$params['icon'] = 'search';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_PREVIEW_ALT_TITLE;
		}
		
		$this->addCustomItem($params);
		
		return $this;
	}
	
	public function addSeparator($params = array()) 
	{
		$params['type'] = 'separator';
		
		$this->addCustomItem($params);
		
		return $this;
	}
	
	protected function renderItems()
	{	
		$content = '';
		
		foreach($this->items AS $item) 
		{			
			$content .= $item->render();
		}
		
		return  $content;
	}	
}